/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

#include "dt-tmp-readconfig.h"
#include <QDebug>
#include <QFile>
#include <QString>
#include <QStringList>
#include <QStandardPaths>
#include <QGSettings>
#include <QMetaEnum>
#include <QCoreApplication>
#include "platformthemedebug.h"



#define ThemeConfigPath "/usr/share/config/globaltheme/token/"
#define DefaultConfigName "kdefault-light.css"

namespace UKUIGlobalDTConfig {

class GlobalDTConfigPrivate
{
public:
    explicit GlobalDTConfigPrivate();
    ~GlobalDTConfigPrivate();

    inline bool getColorValue(QColor &color, const QString key) const;
    inline bool getValue(int &value, const QString key) const;
    inline QColor stringToColor(QString s) const;

    bool load(const QString &cfg);
    void initUKUIGlobalThemeParameters();
    void initThemeConfigPropertyInterface();


    //qt palette
    QColor windowTextActive         = QColor(38, 38, 38, 1 * 255);
    QColor windowTextInactive       = QColor(115, 115, 115, 1 * 255);
    QColor windowTextDisable        = QColor(75, 175, 175, 1 * 255);
    QColor buttonActive             = QColor(230, 230, 230, 1 * 255);
    QColor buttonInactive           = QColor(230, 230, 230, 1 * 255);
    QColor buttonDisable            = QColor(238, 238, 238, 1 * 255);
    QColor lightActive              = QColor(255, 255, 255, 1 * 255);
    QColor lightInactive            = QColor(255, 255, 255, 1 * 255);
    QColor lightDisable             = QColor(250, 250, 250, 1 * 255);
    QColor midlightActive           = QColor(220, 220, 220, 1 * 255);
    QColor midlightInactive         = QColor(220, 220, 220, 1 * 255);
    QColor midlightDisable          = QColor(230, 230, 230, 1 * 255);
    QColor darkActive               = QColor(0, 0, 0, 1 * 255);
    QColor darkInactive             = QColor(0, 0, 0, 1 * 255);
    QColor darkDisable              = QColor(64, 64, 64, 1 * 255);
    QColor midActive                = QColor(115, 115, 115, 1 * 255);
    QColor midInactive              = QColor(115, 115, 115, 1 * 255);
    QColor midDisable               = QColor(102, 102, 102, 1 * 255);
    QColor textActive               = QColor(38, 38, 38, 1 * 255);
    QColor textInactive             = QColor(38, 38, 38, 1 * 255);
    QColor textDisable              = QColor(175, 175, 175, 1 * 255);
    QColor brightTextActive         = QColor(0, 0, 0, 1 * 255);
    QColor brightTextInactive       = QColor(0, 0, 0, 1 * 255);
    QColor brightTextDisable        = QColor(0, 0, 0, 1 * 255);
    QColor buttonTextActive         = QColor(38, 38, 38, 1 * 255);
    QColor buttonTextInactive       = QColor(38, 38, 38, 1 * 255);
    QColor buttonTextDisable        = QColor(175, 175, 175, 1 * 255);
    QColor baseActive               = QColor(255, 255, 255, 1 * 255);
    QColor baseInactive             = QColor(250, 250, 250, 1 * 255);
    QColor baseDisable              = QColor(238, 238, 238, 1 * 255);
    QColor windowActive             = QColor(246, 246, 246, 1 * 255);
    QColor windowInactive           = QColor(242, 242, 242, 1 * 255);
    QColor windowDisable            = QColor(238, 238, 238, 1 * 255);
    QColor shadowActive             = QColor(0, 0, 0, 0.4 * 255);
    QColor shadowInactive           = QColor(0, 0, 0, 0.25 * 255);
    QColor shadowDisable            = QColor(0, 0, 0, 0.25 * 255);
    QColor highLightActive          = QColor(55, 144, 250, 1 * 255);
    QColor highLightInactive        = QColor(55, 144, 250, 1 * 255);
    QColor highLightDisable         = QColor(238, 238, 238, 1 * 255);
    QColor highLightedTextActive    = QColor(255, 255, 255, 1 * 255);
    QColor highLightedTextInactive  = QColor(255, 255, 255, 1 * 255);
    QColor highLightedTextDisable   = QColor(175, 175, 175, 1 * 255);
    QColor linkActive               = QColor(55, 144, 250, 1 * 255);
    QColor linkInactive             = QColor(55, 144, 250, 1 * 255);
    QColor linkDisable              = QColor(175, 175, 175, 1 * 255);
    QColor linkVisitedActive        = QColor();
    QColor linkVisitedInactive      = QColor();
    QColor linkVisitedDisable       = QColor(175, 175, 175, 1 * 255);
    QColor alternateBaseActive      = QColor(242, 242, 242, 1 * 255);
    QColor alternateBaseInactive    = QColor(242, 242, 242, 1 * 255);
    QColor alternateBaseDisable     = QColor(242, 242, 242, 1 * 255);
    QColor noRoleActive             = QColor(250, 250, 250, 1 * 255);
    QColor noRoleInactive           = QColor(250, 250, 250, 1 * 255);
    QColor noRoleDisable            = QColor(235, 235, 235, 1 * 255);
    QColor toolTipBaseActive        = QColor(255, 255, 255, 1 * 255);
    QColor toolTipBaseInactive      = QColor(255, 255, 255, 1 * 255);
    QColor toolTipBaseDisable       = QColor(255, 255, 255, 1 * 255);
    QColor toolTipTextActive        = QColor(38, 38, 38, 1 * 255);
    QColor toolTipTextInactive      = QColor(38, 38, 38, 1 * 255);
    QColor toolTipTextDisable       = QColor(38, 38, 38, 1 * 255);
    QColor placeholderTextActive    = QColor(165, 165, 165, 1 * 255);
    QColor placeholderTextInactive  = QColor(165, 165, 165, 1 * 255);
    QColor placeholderTextDisable   = QColor(200, 200, 200, 1 * 255);

    //custom palette
    QColor kComponentNormal                 = QColor(230, 230, 230, 1 * 255);
    QColor kLineBrandDisable                = QColor(55, 144, 250, 0 * 255);
    QColor kFontWhiteSecondary              = QColor(255, 255, 255, 0.65 * 255);
    QColor kGray12                          = QColor(165, 165, 165, 1 * 255);
    QColor kContainSecondaryAlpphaNormal    = QColor(0, 0, 0, 0.03 * 255);
    QColor kGray16                          = QColor(64, 64, 64, 1 * 255);
    QColor kBrand1                          = QColor(55, 144, 250, 1 * 255);
    QColor kLineComponentNormal             = QColor(0, 0, 0, 0 * 255);
    QColor kLineBrandClick                  = QColor(55, 144, 250, 0 * 255);
    QColor kGray3                           = QColor(242, 242, 242, 1 * 255);
    QColor kGray0                           = QColor(255, 255, 255, 1 * 255);
    QColor kLineDisable                     = QColor(0, 0, 0, 0.05 * 255);
    QColor kGrayAlpha2                      = QColor(0, 0, 0, 0.08 * 255);
    QColor kGrayAlpha0                      = QColor(0, 0, 0, 0 * 255);
    QColor kBrand2                          = QColor(114, 46, 209, 1 * 255);
    QColor kLineNormal                      = QColor(0, 0, 0, 0.1 * 255);
    QColor kGrayAlpha5                      = QColor(0, 0, 0, 0.18 * 255);
    QColor kGrayAlpha12                     = QColor(0, 0, 0, 0.03 * 255);
    QColor kErrorClick                      = QColor();
    QColor kLineWindowAcitive               = QColor(0, 0, 0, 0.1 * 255);
    QColor kComponentAlphaClick             = QColor(0, 0, 0, 0.2 * 255);
    QColor kBrand3                          = QColor(235, 48, 150, 1 * 255);
    QColor kSuccessNormal                   = QColor(82, 196, 41, 1 * 255);
    QColor kGrayAlpha1                      = QColor(0, 0, 0, 0.05 * 255);
    QColor kFontWhiteSecondaryDisable       = QColor(255, 255, 255, 0.3 * 255);
    QColor kGray4                           = QColor(238, 238, 238, 1 * 255);
    QColor kContainClick                    = QColor(238, 238, 238, 1 * 255);
    QColor kGray8                           = QColor(210, 210, 210, 1 * 255);
    QColor kGrayAlpha4                      = QColor(0, 0, 0, 0.15 * 255);
    QColor kModalmask                       = QColor(0, 0, 0, 0.2 * 255);
    QColor kLineBrandNormal                 = QColor(55, 144, 250, 0 * 255);
    QColor kBrand5                          = QColor(0, 129, 31, 1 * 255);
    QColor kGray10                          = QColor(185, 185, 185, 1 * 255);
    QColor kWarningNormal                   = QColor(246, 140, 39, 1 * 255);
    QColor kGray9                           = QColor(200, 200, 200, 1 * 255);
    QColor kGray1                           = QColor(250, 250, 250, 1 * 255);
    QColor kGray14                          = QColor(102, 102, 102, 1 * 255);
    QColor kLineComponentClick              = QColor(0, 0, 0, 0 * 255);
    QColor kWhite                           = QColor(255, 255, 255, 1 * 255);
    QColor kGray7                           = QColor(220, 220, 220, 1 * 255);
    QColor kBrandFocus                      = QColor();
    QColor kGrayAlpha9                      = QColor(0, 0, 0, 0.3 * 255);
    QColor kGrayAlpha10                     = QColor(0, 0, 0, 0.35 * 255);
    QColor kGrayAlpha6                      = QColor(0, 0, 0, 0.2 * 255);
    QColor kContainHover                    = QColor(242, 242, 242, 1 * 255);
    QColor kLineComponentDisable            = QColor(0, 0, 0, 0 * 255);
    QColor kFontWhite                       = QColor(255, 255, 255, 1 * 255);
    QColor kGrayAlpha11                     = QColor(0, 0, 0, 0.55 * 255);
    QColor kGray6                           = QColor(230, 230, 230, 1 * 255);
    QColor kFontWhiteDisable                = QColor(255, 255, 255, 0.35 * 255);
    QColor kComponentDisable                = QColor(238, 238, 238, 1 * 255);
    QColor kBlack                           = QColor(0, 0, 0, 1 * 255);
    QColor kComponentAlphaDisable           = QColor(0, 0, 0, 0.08 * 255);
    QColor kFontSecondaryDisable            = QColor(185, 185, 185, 1 * 255);
    QColor kGrayAlpha8                      = QColor(0, 0, 0, 0.28 * 255);
    QColor kFontPrimaryDisable              = QColor(175, 175, 175, 1 * 255);
    QColor kGray5                           = QColor(235, 235, 235, 1 * 255);
    QColor kFontSecondary                   = QColor(115, 115, 115, 1 * 255);
    QColor kErrorNormal                     = QColor(243, 34, 45, 1 * 255);
    QColor kComponentHover                  = QColor(220, 220, 220, 1 * 255);
    QColor kComponentAlphaHover             = QColor(0, 0, 0, 0.15 * 255);
    QColor kGrayAlpha3                      = QColor(0, 0, 0, 0.1 * 255);
    QColor kGray13                          = QColor(115, 115, 115, 1 * 255);
    QColor kGray11                          = QColor(175, 175, 175, 1 * 255);
    QColor kErrorHover                      = QColor();
    QColor kContainGeneralNormal            = QColor(255, 255, 255, 1 * 255);
    QColor kComponentAlphaNormal            = QColor(0, 0, 0, 0.1 * 255);
    QColor kGray15                          = QColor(77, 77, 77, 1 * 255);
    QColor kBrand7                          = QColor(241, 133, 43, 1 * 255);
    QColor kGrayAlpha7                      = QColor(0, 0, 0, 0.25 * 255);
    QColor kLineBrandHover                  = QColor(55, 144, 250, 0 * 255);
    QColor kGrayAlpha13                     = QColor(0, 0, 0, 0.4 * 255);
    QColor kLineComponentHover              = QColor(0, 0, 0, 0 * 255);
    QColor kBrand6                          = QColor(223, 49, 55, 1 * 255);
    QColor kGray17                          = QColor(38, 38, 38, 1 * 255);
    QColor kContainGeneralAlphaClick        = QColor(0, 0, 0, 0.08 * 255);
    QColor kComponentClick                  = QColor(185, 185, 185, 1 * 255);
    QColor kBrandClick                      = QColor();
    QColor kFontStrong                      = QColor(0, 0, 0, 1 * 255);
    QColor kLineWindow                      = QColor(0, 0, 0, 0.1 * 255);
    QColor kGray2                           = QColor(246, 246, 246, 1 * 255);
    QColor kBrand4                          = QColor(240, 188, 54, 1 * 255);
    QColor kContainSecondaryNormal          = QColor(246, 246, 246, 1 * 255);
    QColor kBrandHover                      = QColor();
    QColor kContainGeneralAlphaHover        = QColor(0, 0, 0, 0.05 * 255);
    QColor kFontPrimary                     = QColor(38, 38, 38, 1 * 255);
    QColor kBrandNormal                     = QColor(55, 144, 250, 1 * 255);

    //line and margin
    int normalline      = 1;
    int focusline       = 2;
    int kmarginMin      = 4;
    int kmarginNormal   = 8;
    int kmarginBig      = 16;
    int kmarginContent  = 40;
    int kmagrinWindow   = 24;

    //radius
    int kradiusMin      = 4;
    int kradiusMenu     = 8;
    int kradiusNormal   = 6;
    int kradiusWindow   = 12;

    bool m_ukui_dt_loadConfig = false;
    QString m_cfgPath;
    QString m_widgetThemeName = "default";
    QString m_lightMode = "light";
    QString m_ukui_dt_path;
    QString m_dt_configData;
};

GlobalDTConfigPrivate::GlobalDTConfigPrivate()
{
    if (QGSettings::isSchemaInstalled("org.ukui.style")) {
        QGSettings *settings = new QGSettings("org.ukui.style");

        QString styleName = settings->get("styleName").toString();
        if (styleName == "ukui-light" || styleName == "ukui-white") {
            m_lightMode = "light";
        } else if (styleName == "ukui-dark" || styleName == "ukui-black") {
            m_lightMode = "dark";
        }

        m_widgetThemeName = settings->get("widgetThemeName").toString();
    }

    QString path = ThemeConfigPath;
    //dt config
    m_ukui_dt_path = (QFile::exists(path + "k" + m_widgetThemeName + "-" + m_lightMode + ".css")) ?
                path + "k" + m_widgetThemeName + "-" + m_lightMode + ".css" :
                path + "k" + m_widgetThemeName + ".css";

    m_ukui_dt_loadConfig = this->load(m_ukui_dt_path);

    initUKUIGlobalThemeParameters();
    initThemeConfigPropertyInterface();
}

GlobalDTConfigPrivate::~GlobalDTConfigPrivate()
{

}


bool GlobalDTConfigPrivate::load(const QString &cfg)
{
    if (!QFile::exists(cfg)) {
        qWarning() << "TMPReadConfig load file not exists!" << cfg;
        return false;
    }

    QFile file(cfg);
    if(!file.open(QIODevice::ReadOnly)) {
        qWarning() << "TMPReadConfig load file Error!" << cfg;
        return false;
    }

    QByteArray configData = file.readAll();
    file.close();
    m_cfgPath = cfg;
    m_dt_configData = configData;

    return true;
}

QColor GlobalDTConfigPrivate::stringToColor(QString s) const
{
    QColor c = Qt::NoBrush;

    QStringList list = s.split(",");
    if (list.length() >= 3) {
        c.setRed(list[0].toInt());
        c.setGreen(list[1].toInt());
        c.setBlue(list[2].toInt());
        if (list.length() == 4) {
            QStringList alphaList = QString(list[3]).split("*");
            if (alphaList.length() == 2) {
                if (alphaList[0].toFloat() == 255.0)
                    c.setAlphaF(alphaList[1].toFloat());
                else if (alphaList[1].toFloat() == 255.0)
                    c.setAlphaF(alphaList[0].toFloat());
                else
                    c.setAlphaF(alphaList[0].toFloat() * alphaList[1].toFloat() / 255.0);

            } else if (alphaList.length() == 1) {
                if(alphaList[0].toFloat() <= 1.0)
                    c.setAlphaF(alphaList[0].toFloat());
                else
                    c.setAlphaF(alphaList[0].toFloat()/255.0);
            } else
                c.setAlphaF(1.0);
        }
    } else if (list.length() == 1 && list[0].startsWith("#")) {
        c = list[0];
    }

    return c;
}

inline bool GlobalDTConfigPrivate::getColorValue(QColor &c, const QString key) const
{

    if (m_dt_configData.isEmpty()) {
        qWarning() << "get fills array isempty";
        return false;
    }

    if (!m_dt_configData.contains("--" + key)) {
        qWarning() << "Unvalid key!" << key;
        return false;
    }

    int start_index, end_index;
    QString color;

    //get value type
    start_index = m_dt_configData.indexOf("--" + key);
    if (start_index == -1) {
        qWarning() << "Config file format error!" << key;
        return false;
    }
    start_index += QString("--" + key).length();
    end_index = m_dt_configData.indexOf(")", start_index);
    color = m_dt_configData.mid(start_index, end_index - start_index);


    if (color.contains(": var(")) {
        //quote key
        start_index = color.indexOf(": var(--");
        if (start_index == -1) {
            qWarning() << "Config file format error!" << key;
            return false;
        }
        start_index += QString(": var(--").length();
        end_index = color.indexOf(")", start_index);
        getColorValue(c, color.mid(start_index, end_index - start_index).toLower());

    } else if (color.contains(": rgba(")) {

        start_index = color.indexOf(": rgba(");
        if (start_index == -1) {
            qWarning() << "Config file format error!" << key;
            return false;
        }
        start_index += QString(": rgba(").length();
        end_index = color.indexOf(")", start_index);
        c = stringToColor(color.mid(start_index, end_index - start_index));

    } else {
        qWarning() << "Get key type error!" << key;
        return false;
    }

    return true;
}

inline bool GlobalDTConfigPrivate::getValue(int &value, const QString key) const
{
    if (m_dt_configData.isEmpty()) {
        qWarning() << "get fills array isempty!" << key;
        return false;
    }

    if (!m_dt_configData.contains(key)) {
        qWarning() << "Unvalid key!" << key;
        return false;
    }

    int start_index = m_dt_configData.indexOf("--" + key + ": ");
    if (start_index == -1) {
        qWarning() << "Config file format error!" << key;
        return false;
    }
    start_index += QString("--" + key + ": ").length();
    int end_index = m_dt_configData.indexOf("px", start_index);

    value = m_dt_configData.mid(start_index, end_index - start_index).toInt();

    return true;
}

void GlobalDTConfigPrivate::initUKUIGlobalThemeParameters()
{
    if (m_ukui_dt_loadConfig) {
        //default palette
        getColorValue(windowTextActive          , "windowtext-active");
        getColorValue(windowTextInactive        , "windowtext-inactive");
        getColorValue(windowTextDisable         , "windowtext-disable");
        getColorValue(buttonActive              , "button-active");
        getColorValue(buttonInactive            , "button-inactive");
        getColorValue(buttonDisable             , "button-disable");
        getColorValue(lightActive               , "light-active");
        getColorValue(lightInactive             , "light-inactive");
        getColorValue(lightDisable              , "light-disable");
        getColorValue(midlightActive            , "midlight-active");
        getColorValue(midlightInactive          , "midlight-inactive");
        getColorValue(midlightDisable           , "midlight-disable");
        getColorValue(darkActive                , "dark-active");
        getColorValue(darkInactive              , "dark-inactive");
        getColorValue(darkDisable               , "dark-disable");
        getColorValue(midActive                 , "mid-active");
        getColorValue(midInactive               , "mid-inactive");
        getColorValue(midDisable                , "mid-disable");
        getColorValue(textActive                , "text-active");
        getColorValue(textInactive              , "text-inactive");
        getColorValue(textDisable               , "text-disable");
        getColorValue(brightTextActive          , "brighttext-active");
        getColorValue(brightTextInactive        , "brighttext-inactive");
        getColorValue(brightTextDisable         , "brighttext-disable");
        getColorValue(buttonTextActive          , "buttontext-active");
        getColorValue(buttonTextInactive        , "buttontext-inactive");
        getColorValue(buttonTextDisable         , "buttontext-disable");
        getColorValue(baseActive                , "base-active");
        getColorValue(baseInactive              , "base-inactive");
        getColorValue(baseDisable               , "base-disable");
        getColorValue(windowActive              , "window-active");
        getColorValue(windowInactive            , "window-inactive");
        getColorValue(windowDisable             , "window-disable");
        getColorValue(shadowActive              , "shadow-active");
        getColorValue(shadowInactive            , "shadow-inactive");
        getColorValue(shadowDisable             , "shadow-disable");
        getColorValue(highLightActive           , "highlight-active");
        getColorValue(highLightInactive         , "highlight-inactive");
        getColorValue(highLightDisable          , "highlight-disable");
        getColorValue(highLightedTextActive     , "highlightedtext-active");
        getColorValue(highLightedTextInactive   , "highlightedtext-inactive");
        getColorValue(highLightedTextDisable    , "highlightedtext-disable");
        getColorValue(linkActive                , "link-active");
        getColorValue(linkInactive              , "link-inactive");
        getColorValue(linkDisable               , "link-disable");
        getColorValue(linkVisitedActive         , "linkvisited-active");
        getColorValue(linkVisitedInactive       , "linkvisited-inactive");
        getColorValue(linkVisitedDisable        , "linkvisited-disable");
        getColorValue(alternateBaseActive       , "alternatebase-active");
        getColorValue(alternateBaseInactive     , "alternatebase-inactive");
        getColorValue(alternateBaseDisable      , "alternatebase-disable");
        getColorValue(noRoleActive              , "norole-active");
        getColorValue(noRoleInactive            , "norole-inactive");
        getColorValue(noRoleDisable             , "norole-disable");
        getColorValue(toolTipBaseActive         , "tooltipbase-active");
        getColorValue(toolTipBaseInactive       , "tooltipbase-inactive");
        getColorValue(toolTipBaseDisable        , "tooltipbase-disable");
        getColorValue(toolTipTextActive         , "tooltiptext-active");
        getColorValue(toolTipTextInactive       , "tooltiptext-inactive");
        getColorValue(toolTipTextDisable        , "tooltiptext-disable");
        getColorValue(placeholderTextActive     , "placeholdertext-active");
        getColorValue(placeholderTextInactive   , "placeholdertext-inactive");
        getColorValue(placeholderTextDisable    , "placeholdertext-disable");

        //custom palette
        getColorValue(kBlack                        , "kblack");
        getColorValue(kBrand1                       , "kbrand1");
        getColorValue(kBrand2                       , "kbrand2");
        getColorValue(kBrand3                       , "kbrand3");
        getColorValue(kBrand4                       , "kbrand4");
        getColorValue(kBrand5                       , "kbrand5");
        getColorValue(kBrand6                       , "kbrand6");
        getColorValue(kBrand7                       , "kbrand7");
        getColorValue(kBrandClick                   , "kbrand-click");
        getColorValue(kBrandFocus                   , "kbrand-focus");
        getColorValue(kBrandHover                   , "kbrand-hover");
        getColorValue(kBrandNormal                  , "kbrand-normal");
        getColorValue(kComponentAlphaClick          , "kcomponent-alpha-click");
        getColorValue(kComponentAlphaDisable        , "kcomponent-alpha-disable");
        getColorValue(kComponentAlphaHover          , "kcomponent-alpha-hover");
        getColorValue(kComponentAlphaNormal         , "kcomponent-alpha-normal");
        getColorValue(kComponentClick               , "kcomponent-click");
        getColorValue(kComponentDisable             , "kcomponent-disable");
        getColorValue(kComponentHover               , "kcomponent-hover");
        getColorValue(kComponentNormal              , "kcomponent-normal");
        getColorValue(kContainClick                 , "kcontain-click");
        getColorValue(kContainHover                 , "kcontain-hover");
        getColorValue(kContainGeneralAlphaClick     , "kcontain-general-alpha-click");
        getColorValue(kContainGeneralAlphaHover     , "kcontain-general-alpha-hover");
        getColorValue(kContainGeneralNormal         , "kcontain-general-normal");
        getColorValue(kContainSecondaryAlpphaNormal , "kcontain-secondary-alpha-normal");
        getColorValue(kContainSecondaryNormal       , "kcontain-secondary-normal");
        getColorValue(kErrorClick                   , "kerror-click");
        getColorValue(kErrorHover                   , "kerror-hover");
        getColorValue(kErrorNormal                  , "kerror-normal");
        getColorValue(kFontPrimary                  , "kfont-primary");
        getColorValue(kFontPrimaryDisable           , "kfont-primary-disable");
        getColorValue(kFontSecondary                , "kfont-secondary");
        getColorValue(kFontSecondaryDisable         , "kfont-secondary-disable");
        getColorValue(kFontStrong                   , "kfont-strong");
        getColorValue(kFontWhite                    , "kfont-white");
        getColorValue(kFontWhiteDisable             , "kfont-white-disable");
        getColorValue(kFontWhiteSecondary           , "kfont-white-secondary");
        getColorValue(kFontWhiteSecondaryDisable    , "kfont-white-secondary-disable");
        getColorValue(kGray0                        , "kgray-0");
        getColorValue(kGray1                        , "kgray-1");
        getColorValue(kGray2                        , "kgray-2");
        getColorValue(kGray3                        , "kgray-3");
        getColorValue(kGray4                        , "kgray-4");
        getColorValue(kGray5                        , "kgray-5");
        getColorValue(kGray6                        , "kgray-6");
        getColorValue(kGray7                        , "kgray-7");
        getColorValue(kGray8                        , "kgray-8");
        getColorValue(kGray9                        , "kgray-9");
        getColorValue(kGray10                       , "kgray-10");
        getColorValue(kGray11                       , "kgray-11");
        getColorValue(kGray12                       , "kgray-12");
        getColorValue(kGray13                       , "kgray-13");
        getColorValue(kGray14                       , "kgray-14");
        getColorValue(kGray15                       , "kgray-15");
        getColorValue(kGray16                       , "kgray-16");
        getColorValue(kGray17                       , "kgray-17");
        getColorValue(kGrayAlpha0                   , "kgray-alpha0");
        getColorValue(kGrayAlpha1                   , "kgray-alpha1");
        getColorValue(kGrayAlpha2                   , "kgray-alpha2");
        getColorValue(kGrayAlpha3                   , "kgray-alpha3");
        getColorValue(kGrayAlpha4                   , "kgray-alpha4");
        getColorValue(kGrayAlpha5                   , "kgray-alpha5");
        getColorValue(kGrayAlpha6                   , "kgray-alpha6");
        getColorValue(kGrayAlpha7                   , "kgray-alpha7");
        getColorValue(kGrayAlpha8                   , "kgray-alpha8");
        getColorValue(kGrayAlpha9                   , "kgray-alpha9");
        getColorValue(kGrayAlpha10                  , "kgray-alpha10");
        getColorValue(kGrayAlpha11                  , "kgray-alpha11");
        getColorValue(kGrayAlpha12                  , "kgray-alpha12");
        getColorValue(kGrayAlpha13                  , "kgray-alpha13");
        getColorValue(kLineBrandClick               , "kline-brand-click");
        getColorValue(kLineBrandDisable             , "kline-brand-disable");
        getColorValue(kLineBrandHover               , "kline-brand-hover");
        getColorValue(kLineBrandNormal              , "kline-brand-normal");
        getColorValue(kLineComponentClick           , "kline-component-click");
        getColorValue(kLineComponentDisable         , "kline-component-disable");
        getColorValue(kLineComponentHover           , "kline-component-hover");
        getColorValue(kLineComponentNormal          , "kline-component-normal");
        getColorValue(kLineDisable                  , "kline-disable");
        getColorValue(kLineNormal                   , "kline-normal");
        getColorValue(kLineWindow                   , "kline-window");
        getColorValue(kLineWindowAcitive            , "kline-window-active");
        getColorValue(kModalmask                    , "kmodalmask");
        getColorValue(kSuccessNormal                , "ksuccess-normal");
        getColorValue(kWarningNormal                , "kwarning-normal");
        getColorValue(kWhite                        , "kwhite");

        //line and margin
        getValue(normalline     , "normalline");
        getValue(focusline      , "focusline");
        getValue(kmarginMin     , "kmargin-min");
        getValue(kmarginNormal  , "kmargin-normal");
        getValue(kmarginBig     , "kmargin-big");
        getValue(kmarginContent , "kmargin-content");
        getValue(kmagrinWindow  , "kmagrin-window");

        //radius
        getValue(kradiusMin     , "kradius-min");
        getValue(kradiusMenu    , "kradius-menu");
        getValue(kradiusNormal  , "kradius-normal");
        getValue(kradiusWindow  , "kradius-window");
    }
}

void GlobalDTConfigPrivate::initThemeConfigPropertyInterface()
{
    //default palette
    qApp->setProperty("windowtext-active",           windowTextActive);
    qApp->setProperty("windowtext-inactive",         windowTextInactive);
    qApp->setProperty("windowtext-disable",          windowTextDisable);
    qApp->setProperty("button-active",               buttonActive);
    qApp->setProperty("button-inactive",             buttonInactive);
    qApp->setProperty("button-disable",              buttonDisable);
    qApp->setProperty("light-active",                lightActive);
    qApp->setProperty("light-inactive",              lightInactive);
    qApp->setProperty("light-disable",               lightDisable);
    qApp->setProperty("midlight-active",             midlightActive);
    qApp->setProperty("midlight-inactive",           midlightInactive);
    qApp->setProperty("midlight-disable",            midlightDisable);
    qApp->setProperty("dark-active",                 darkActive);
    qApp->setProperty("dark-inactive",               darkInactive);
    qApp->setProperty("dark-disable",                darkDisable);
    qApp->setProperty("mid-active",                  midActive);
    qApp->setProperty("mid-inactive",                midInactive);
    qApp->setProperty("mid-disable",                 midDisable);
    qApp->setProperty("text-active",                 textActive);
    qApp->setProperty("text-inactive",               textInactive);
    qApp->setProperty("text-disable",                textDisable);
    qApp->setProperty("brighttext-active",           brightTextActive);
    qApp->setProperty("brighttext-inactive",         brightTextInactive);
    qApp->setProperty("brighttext-disable",          brightTextDisable);
    qApp->setProperty("buttontext-active",           buttonTextActive);
    qApp->setProperty("buttontext-inactive",         buttonTextInactive);
    qApp->setProperty("buttontext-disable",          buttonTextDisable);
    qApp->setProperty("base-active",                 baseActive);
    qApp->setProperty("base-inactive",               baseInactive);
    qApp->setProperty("base-disable",                baseDisable);
    qApp->setProperty("window-active",               windowActive);
    qApp->setProperty("window-inactive",             windowInactive);
    qApp->setProperty("window-disable",              windowDisable);
    qApp->setProperty("shadow-active",               shadowActive);
    qApp->setProperty("shadow-inactive",             shadowInactive);
    qApp->setProperty("shadow-disable",              shadowDisable);
    qApp->setProperty("highlight-active",            highLightActive);
    qApp->setProperty("highlight-inactive",          highLightInactive);
    qApp->setProperty("highlight-disable",           highLightDisable);
    qApp->setProperty("highlightedtext-active",      highLightedTextActive);
    qApp->setProperty("highlightedtext-inactive",    highLightedTextInactive);
    qApp->setProperty("highlightedtext-disable",     highLightedTextDisable);
    qApp->setProperty("link-active",                 linkActive);
    qApp->setProperty("link-inactive",               linkInactive);
    qApp->setProperty("link-disable",                linkDisable);
    qApp->setProperty("linkvisited-active",          linkVisitedActive);
    qApp->setProperty("linkvisited-inactive",        linkVisitedInactive);
    qApp->setProperty("linkvisited-disable",         linkVisitedDisable);
    qApp->setProperty("alternatebase-active",        alternateBaseActive);
    qApp->setProperty("alternatebase-inactive",      alternateBaseInactive);
    qApp->setProperty("alternatebase-disable",       alternateBaseDisable);
    qApp->setProperty("norole-active",               noRoleActive);
    qApp->setProperty("norole-inactive",             noRoleInactive);
    qApp->setProperty("norole-disable",              noRoleDisable);
    qApp->setProperty("tooltipbase-active",          toolTipBaseActive);
    qApp->setProperty("tooltipbase-inactive",        toolTipBaseInactive);
    qApp->setProperty("tooltipbase-disable",         toolTipBaseDisable);
    qApp->setProperty("tooltiptext-active",          toolTipTextActive);
    qApp->setProperty("tooltiptext-inactive",        toolTipTextInactive);
    qApp->setProperty("tooltiptext-disable",         toolTipTextDisable);
    qApp->setProperty("placeholdertext-active",      placeholderTextActive);
    qApp->setProperty("placeholdertext-inactive",    placeholderTextInactive);
    qApp->setProperty("placeholdertext-disable",     placeholderTextDisable);

    //custom palette
    qApp->setProperty("kblack",                            kBlack);
    qApp->setProperty("kbrand1",                           kBrand1);
    qApp->setProperty("kbrand2",                           kBrand2);
    qApp->setProperty("kbrand3",                           kBrand3);
    qApp->setProperty("kbrand4",                           kBrand4);
    qApp->setProperty("kbrand5",                           kBrand5);
    qApp->setProperty("kbrand6",                           kBrand6);
    qApp->setProperty("kbrand7",                           kBrand7);
    qApp->setProperty("kbrand-click",                      kBrandClick);
    qApp->setProperty("kbrand-focus",                      kBrandFocus);
    qApp->setProperty("kbrand-hover",                      kBrandHover);
    qApp->setProperty("kbrand-normal",                     kBrandNormal);
    qApp->setProperty("kcomponent-alpha-click",            kComponentAlphaClick);
    qApp->setProperty("kcomponent-alpha-disable",          kComponentAlphaDisable);
    qApp->setProperty("kcomponent-alpha-hover",            kComponentAlphaHover);
    qApp->setProperty("kcomponent-alpha-normal",           kComponentAlphaNormal);
    qApp->setProperty("kcomponent-click",                  kComponentClick);
    qApp->setProperty("kcomponent-disable",                kComponentDisable);
    qApp->setProperty("kcomponent-hover",                  kComponentHover);
    qApp->setProperty("kcomponent-normal",                 kComponentNormal);
    qApp->setProperty("kcontain-click",                    kContainClick);
    qApp->setProperty("kcontain-hover",                    kContainHover);
    qApp->setProperty("kcontain-general-alpha-click",      kContainGeneralAlphaClick);
    qApp->setProperty("kcontain-general-alpha-hover",      kContainGeneralAlphaHover);
    qApp->setProperty("kcontain-general-normal",           kContainGeneralNormal);
    qApp->setProperty("kcontain-secondary-alpha-normal",  kContainSecondaryAlpphaNormal);
    qApp->setProperty("kcontain-secondary-normal",         kContainSecondaryNormal);
    qApp->setProperty("kerror-click",                      kErrorClick);
    qApp->setProperty("kerror-hover",                      kErrorHover);
    qApp->setProperty("kerror-normal",                     kErrorNormal);
    qApp->setProperty("kfont-primary",                     kFontPrimary);
    qApp->setProperty("kfont-primary-disable",             kFontPrimaryDisable);
    qApp->setProperty("kfont-secondary",                   kFontSecondary);
    qApp->setProperty("kfont-secondary-disable",           kFontSecondaryDisable);
    qApp->setProperty("kfont-strong",                      kFontStrong);
    qApp->setProperty("kfont-white",                       kFontWhite);
    qApp->setProperty("kfont-white-disable",               kFontWhiteDisable);
    qApp->setProperty("kfont-white-secondary",             kFontWhiteSecondary);
    qApp->setProperty("kfont-white-secondary-disable",     kFontWhiteSecondaryDisable);
    qApp->setProperty("kgray-0",                           kGray0);
    qApp->setProperty("kgray-1",                           kGray1);
    qApp->setProperty("kgray-2",                           kGray2);
    qApp->setProperty("kgray-3",                           kGray3);
    qApp->setProperty("kgray-4",                           kGray4);
    qApp->setProperty("kgray-5",                           kGray5);
    qApp->setProperty("kgray-6",                           kGray6);
    qApp->setProperty("kgray-7",                           kGray7);
    qApp->setProperty("kgray-8",                           kGray8);
    qApp->setProperty("kgray-9",                           kGray9);
    qApp->setProperty("kgray-10",                          kGray10);
    qApp->setProperty("kgray-11",                          kGray11);
    qApp->setProperty("kgray-12",                          kGray12);
    qApp->setProperty("kgray-13",                          kGray13);
    qApp->setProperty("kgray-14",                          kGray14);
    qApp->setProperty("kgray-15",                          kGray15);
    qApp->setProperty("kgray-16",                          kGray16);
    qApp->setProperty("kgray-17",                          kGray17);
    qApp->setProperty("kgray-alpha0",                      kGrayAlpha0);
    qApp->setProperty("kgray-alpha1",                      kGrayAlpha1);
    qApp->setProperty("kgray-alpha2",                      kGrayAlpha2);
    qApp->setProperty("kgray-alpha3",                      kGrayAlpha3);
    qApp->setProperty("kgray-alpha4",                      kGrayAlpha4);
    qApp->setProperty("kgray-alpha5",                      kGrayAlpha5);
    qApp->setProperty("kgray-alpha6",                      kGrayAlpha6);
    qApp->setProperty("kgray-alpha7",                      kGrayAlpha7);
    qApp->setProperty("kgray-alpha8",                      kGrayAlpha8);
    qApp->setProperty("kgray-alpha9",                      kGrayAlpha9);
    qApp->setProperty("kgray-alpha10",                     kGrayAlpha10);
    qApp->setProperty("kgray-alpha11",                     kGrayAlpha11);
    qApp->setProperty("kgray-alpha12",                     kGrayAlpha12);
    qApp->setProperty("kgray-alpha13",                     kGrayAlpha13);
    qApp->setProperty("kline-brand-click",                 kLineBrandClick);
    qApp->setProperty("kline-brand-disable",               kLineBrandDisable);
    qApp->setProperty("kline-brand-hover",                 kLineBrandHover);
    qApp->setProperty("kline-brand-normal",                kLineBrandNormal);
    qApp->setProperty("kline-component-click",             kLineComponentClick);
    qApp->setProperty("kline-component-disable",           kLineComponentDisable);
    qApp->setProperty("kline-component-hover",             kLineComponentHover);
    qApp->setProperty("kline-component-normal",            kLineComponentNormal);
    qApp->setProperty("kline-disable",                     kLineDisable);
    qApp->setProperty("kline-normal",                      kLineNormal);
    qApp->setProperty("kline-window",                      kLineWindow);
    qApp->setProperty("kline-window-active",               kLineWindowAcitive);
    qApp->setProperty("kmodalmask",                        kModalmask);
    qApp->setProperty("ksuccess-normal",                   kSuccessNormal);
    qApp->setProperty("kwarning-normal",                   kWarningNormal);
    qApp->setProperty("kwhite",                            kWhite);

    //line and margin
    qApp->setProperty("normalline",      normalline);
    qApp->setProperty("focusline",       focusline);
    qApp->setProperty("kmargin-min",     kmarginMin);
    qApp->setProperty("kmargin-normal",  kmarginNormal);
    qApp->setProperty("kmargin-big",     kmarginBig);
    qApp->setProperty("kmargin-content", kmarginContent);
    qApp->setProperty("kmagrin-window",  kmagrinWindow);

    //radius
    qApp->setProperty("kradius-min",     kradiusMin);
    qApp->setProperty("kradius-menu",    kradiusMenu);
    qApp->setProperty("kradius-normal",  kradiusNormal);
    qApp->setProperty("kradius-window",  kradiusWindow);
}



GlobalDTConfig::GlobalDTConfig() : d(new GlobalDTConfigPrivate())
{
    if (QGSettings::isSchemaInstalled("org.ukui.style")) {
        auto *settings = new QGSettings("org.ukui.style");
        connect(settings, &QGSettings::changed, this, [=](const QString &key) {
            if (key == "styleName" || key == "widgetThemeName" || key == "themeColor" ||
                    key == "style-name" || key == "widget-theme-name" || key == "theme-color") {
                emit tokenChanged();
            }
        });
    }
}

GlobalDTConfig::~GlobalDTConfig()
{
    if (d) {
        delete d;
        d = nullptr;
    }
}

QColor GlobalDTConfig::windowTextActive() const
{
    return d->windowTextActive;
}

QColor GlobalDTConfig::windowTextInactive() const
{
    return d->windowTextInactive;
}

QColor GlobalDTConfig::windowTextDisable() const
{
    return d->windowTextDisable;
}

QColor GlobalDTConfig::buttonActive() const
{
    return d->buttonActive;
}

QColor GlobalDTConfig::buttonInactive() const
{
    return d->buttonInactive;
}

QColor GlobalDTConfig::buttonDisable() const
{
    return d->buttonDisable;
}

QColor GlobalDTConfig::lightActive() const
{
    return d->lightActive;
}

QColor GlobalDTConfig::lightInactive() const
{
    return d->lightInactive;
}

QColor GlobalDTConfig::lightDisable() const
{
    return d->lightDisable;
}

QColor GlobalDTConfig::midlightActive() const
{
    return d->midlightActive;
}

QColor GlobalDTConfig::midlightInactive() const
{
    return d->midlightInactive;
}

QColor GlobalDTConfig::midlightDisable() const
{
    return d->midlightDisable;
}

QColor GlobalDTConfig::darkActive() const
{
    return d->darkActive;
}

QColor GlobalDTConfig::darkInactive() const
{
    return d->darkInactive;
}

QColor GlobalDTConfig::darkDisable() const
{
    return d->darkDisable;
}

QColor GlobalDTConfig::midActive() const
{
    return d->midActive;
}

QColor GlobalDTConfig::midInactive() const
{
    return d->midInactive;
}

QColor GlobalDTConfig::midDisable() const
{
    return d->midDisable;
}

QColor GlobalDTConfig::textActive() const
{
    return d->textActive;
}

QColor GlobalDTConfig::textInactive() const
{
    return d->textInactive;
}

QColor GlobalDTConfig::textDisable() const
{
    return d->textDisable;
}

QColor GlobalDTConfig::brightTextActive() const
{
    return d->brightTextActive;
}

QColor GlobalDTConfig::brightTextInactive() const
{
    return d->brightTextInactive;
}

QColor GlobalDTConfig::brightTextDisable() const
{
    return d->brightTextDisable;
}

QColor GlobalDTConfig::buttonTextActive() const
{
    return d->buttonTextActive;
}

QColor GlobalDTConfig::buttonTextInactive() const
{
    return d->buttonTextInactive;
}

QColor GlobalDTConfig::buttonTextDisable() const
{
    return d->buttonTextDisable;
}

QColor GlobalDTConfig::baseActive() const
{
    return d->baseActive;
}

QColor GlobalDTConfig::baseInactive() const
{
    return d->baseInactive;
}

QColor GlobalDTConfig::baseDisable() const
{
    return d->baseDisable;
}

QColor GlobalDTConfig::windowActive() const
{
    return d->windowActive;
}

QColor GlobalDTConfig::windowInactive() const
{
    return d->windowInactive;
}

QColor GlobalDTConfig::windowDisable() const
{
    return d->windowDisable;
}

QColor GlobalDTConfig::shadowActive() const
{
    return d->shadowActive;
}

QColor GlobalDTConfig::shadowInactive() const
{
    return d->shadowInactive;
}

QColor GlobalDTConfig::shadowDisable() const
{
    return d->shadowDisable;
}

QColor GlobalDTConfig::highLightActive() const
{
    return d->highLightActive;
}

QColor GlobalDTConfig::highLightInactive() const
{
    return d->highLightInactive;
}

QColor GlobalDTConfig::highLightDisable() const
{
    return d->highLightDisable;
}

QColor GlobalDTConfig::highLightedTextActive() const
{
    return d->highLightedTextActive;
}

QColor GlobalDTConfig::highLightedTextInactive() const
{
    return d->highLightedTextInactive;
}

QColor GlobalDTConfig::highLightedTextDisable() const
{
    return d->highLightedTextDisable;
}

QColor GlobalDTConfig::linkActive() const
{
    return d->linkActive;
}

QColor GlobalDTConfig::linkInactive() const
{
    return d->linkInactive;
}

QColor GlobalDTConfig::linkDisable() const
{
    return d->linkDisable;
}

QColor GlobalDTConfig::linkVisitedActive() const
{
    return d->linkVisitedActive;
}

QColor GlobalDTConfig::linkVisitedInactive() const
{
    return d->linkVisitedInactive;
}

QColor GlobalDTConfig::linkVisitedDisable() const
{
    return d->linkVisitedDisable;
}

QColor GlobalDTConfig::alternateBaseActive() const
{
    return d->alternateBaseActive;
}

QColor GlobalDTConfig::alternateBaseInactive() const
{
    return d->alternateBaseInactive;
}

QColor GlobalDTConfig::alternateBaseDisable() const
{
    return d->alternateBaseDisable;
}

QColor GlobalDTConfig::noRoleActive() const
{
    return d->noRoleActive;
}

QColor GlobalDTConfig::noRoleInactive() const
{
    return d->noRoleInactive;
}

QColor GlobalDTConfig::noRoleDisable() const
{
    return d->noRoleDisable;
}

QColor GlobalDTConfig::toolTipBaseActive() const
{
    return d->toolTipBaseActive;
}

QColor GlobalDTConfig::toolTipBaseInactive() const
{
    return d->toolTipBaseInactive;
}

QColor GlobalDTConfig::toolTipBaseDisable() const
{
    return d->toolTipBaseDisable;
}

QColor GlobalDTConfig::toolTipTextActive() const
{
    return d->toolTipTextActive;
}

QColor GlobalDTConfig::toolTipTextInactive() const
{
    return d->toolTipTextInactive;
}

QColor GlobalDTConfig::toolTipTextDisable() const
{
    return d->toolTipTextDisable;
}

QColor GlobalDTConfig::placeholderTextActive() const
{
    return d->placeholderTextActive;
}

QColor GlobalDTConfig::placeholderTextInactive() const
{
    return d->placeholderTextInactive;
}

QColor GlobalDTConfig::placeholderTextDisable() const
{
    return d->placeholderTextDisable;
}

QColor GlobalDTConfig::kComponentNormal() const
{
    return d->kComponentNormal;
}

QColor GlobalDTConfig::kLineBrandDisable() const
{
    return d->kLineBrandDisable;
}

QColor GlobalDTConfig::kFontWhiteSecondary() const
{
    return d->kFontWhiteSecondary;
}

QColor GlobalDTConfig::kGray12() const
{
    return d->kGray12;
}

QColor GlobalDTConfig::kContainSecondaryAlpphaNormal() const
{
    return d->kContainSecondaryAlpphaNormal;
}

QColor GlobalDTConfig::kGray16() const
{
    return d->kGray16;
}

QColor GlobalDTConfig::kBrand1() const
{
    return d->kBrand1;
}

QColor GlobalDTConfig::kLineComponentNormal() const
{
    return d->kLineComponentNormal;
}

QColor GlobalDTConfig::kLineBrandClick() const
{
    return d->kLineBrandClick;
}

QColor GlobalDTConfig::kGray3() const
{
    return d->kGray3;
}

QColor GlobalDTConfig::kGray0() const
{
    return d->kGray0;
}

QColor GlobalDTConfig::kLineDisable() const
{
    return d->kLineDisable;
}

QColor GlobalDTConfig::kGrayAlpha2() const
{
    return d->kGrayAlpha2;
}

QColor GlobalDTConfig::kGrayAlpha0() const
{
    return d->kGrayAlpha0;
}

QColor GlobalDTConfig::kBrand2() const
{
    return d->kBrand2;
}

QColor GlobalDTConfig::kLineNormal() const
{
    return d->kLineNormal;
}

QColor GlobalDTConfig::kGrayAlpha5() const
{
    return d->kGrayAlpha5;
}

QColor GlobalDTConfig::kGrayAlpha12() const
{
    return d->kGrayAlpha12;
}

QColor GlobalDTConfig::kErrorClick() const
{
    return d->kErrorClick;
}

QColor GlobalDTConfig::kLineWindowAcitive() const
{
    return d->kLineWindowAcitive;
}

QColor GlobalDTConfig::kComponentAlphaClick() const
{
    return d->kComponentAlphaClick;
}

QColor GlobalDTConfig::kBrand3() const
{
    return d->kBrand3;
}

QColor GlobalDTConfig::kSuccessNormal() const
{
    return d->kSuccessNormal;
}

QColor GlobalDTConfig::kGrayAlpha1() const
{
    return d->kGrayAlpha1;
}

QColor GlobalDTConfig::kFontWhiteSecondaryDisable() const
{
    return d->kFontWhiteSecondaryDisable;
}

QColor GlobalDTConfig::kGray4() const
{
    return d->kGray4;
}

QColor GlobalDTConfig::kContainClick() const
{
    return d->kContainClick;
}

QColor GlobalDTConfig::kGray8() const
{
    return d->kGray8;
}

QColor GlobalDTConfig::kGrayAlpha4() const
{
    return d->kGrayAlpha4;
}

QColor GlobalDTConfig::kModalmask() const
{
    return d->kModalmask;
}

QColor GlobalDTConfig::kLineBrandNormal() const
{
    return d->kLineBrandNormal;
}

QColor GlobalDTConfig::kBrand5() const
{
    return d->kBrand5;
}

QColor GlobalDTConfig::kGray10() const
{
    return d->kGray10;
}

QColor GlobalDTConfig::kWarningNormal() const
{
    return d->kWarningNormal;
}

QColor GlobalDTConfig::kGray9() const
{
    return d->kGray9;
}

QColor GlobalDTConfig::kGray1() const
{
    return d->kGray1;
}

QColor GlobalDTConfig::kGray14() const
{
    return d->kGray14;
}

QColor GlobalDTConfig::kLineComponentClick() const
{
    return d->kLineComponentClick;
}

QColor GlobalDTConfig::kWhite() const
{
    return d->kWhite;
}

QColor GlobalDTConfig::kGray7() const
{
    return d->kGray7;
}

QColor GlobalDTConfig::kBrandFocus() const
{
    return d->kBrandFocus;
}

QColor GlobalDTConfig::kGrayAlpha9() const
{
    return d->kGrayAlpha9;
}

QColor GlobalDTConfig::kGrayAlpha10() const
{
    return d->kGrayAlpha10;
}

QColor GlobalDTConfig::kGrayAlpha6() const
{
    return d->kGrayAlpha6;
}

QColor GlobalDTConfig::kContainHover() const
{
    return d->kContainHover;
}

QColor GlobalDTConfig::kLineComponentDisable() const
{
    return d->kLineComponentDisable;
}

QColor GlobalDTConfig::kFontWhite() const
{
    return d->kFontWhite;
}

QColor GlobalDTConfig::kGrayAlpha11() const
{
    return d->kGrayAlpha11;
}

QColor GlobalDTConfig::kGray6() const
{
    return d->kGray6;
}

QColor GlobalDTConfig::kFontWhiteDisable() const
{
    return d->kFontWhiteDisable;
}

QColor GlobalDTConfig::kComponentDisable() const
{
    return d->kComponentDisable;
}

QColor GlobalDTConfig::kBlack() const
{
    return d->kBlack;
}

QColor GlobalDTConfig::kComponentAlphaDisable() const
{
    return d->kComponentAlphaDisable;
}

QColor GlobalDTConfig::kFontSecondaryDisable() const
{
    return d->kFontSecondaryDisable;
}

QColor GlobalDTConfig::kGrayAlpha8() const
{
    return d->kGrayAlpha8;
}

QColor GlobalDTConfig::kFontPrimaryDisable() const
{
    return d->kFontPrimaryDisable;
}

QColor GlobalDTConfig::kGray5() const
{
    return d->kGray5;
}

QColor GlobalDTConfig::kFontSecondary() const
{
    return d->kFontSecondary;
}

QColor GlobalDTConfig::kErrorNormal() const
{
    return d->kErrorNormal;
}

QColor GlobalDTConfig::kComponentHover() const
{
    return d->kComponentHover;
}

QColor GlobalDTConfig::kComponentAlphaHover() const
{
    return d->kComponentAlphaHover;
}

QColor GlobalDTConfig::kGrayAlpha3() const
{
    return d->kGrayAlpha3;
}

QColor GlobalDTConfig::kGray13() const
{
    return d->kGray13;
}

QColor GlobalDTConfig::kGray11() const
{
    return d->kGray11;
}

QColor GlobalDTConfig::kErrorHover() const
{
    return d->kErrorHover;
}

QColor GlobalDTConfig::kContainGeneralNormal() const
{
    return d->kContainGeneralNormal;
}

QColor GlobalDTConfig::kComponentAlphaNormal() const
{
    return d->kComponentAlphaNormal;
}

QColor GlobalDTConfig::kGray15() const
{
    return d->kGray15;
}

QColor GlobalDTConfig::kBrand7() const
{
    return d->kBrand7;
}

QColor GlobalDTConfig::kGrayAlpha7() const
{
    return d->kGrayAlpha7;
}

QColor GlobalDTConfig::kLineBrandHover() const
{
    return d->kLineBrandHover;
}

QColor GlobalDTConfig::kGrayAlpha13() const
{
    return d->kGrayAlpha13;
}

QColor GlobalDTConfig::kLineComponentHover() const
{
    return d->kLineComponentHover;
}

QColor GlobalDTConfig::kBrand6() const
{
    return d->kBrand6;
}

QColor GlobalDTConfig::kGray17() const
{
    return d->kGray17;
}

QColor GlobalDTConfig::kContainGeneralAlphaClick() const
{
    return d->kContainGeneralAlphaClick;
}

QColor GlobalDTConfig::kComponentClick() const
{
    return d->kComponentClick;
}

QColor GlobalDTConfig::kBrandClick() const
{
    return d->kBrandClick;
}

QColor GlobalDTConfig::kFontStrong() const
{
    return d->kFontStrong;
}

QColor GlobalDTConfig::kLineWindow() const
{
    return d->kLineWindow;
}

QColor GlobalDTConfig::kGray2() const
{
    return d->kGray2;
}

QColor GlobalDTConfig::kBrand4() const
{
    return d->kBrand4;
}

QColor GlobalDTConfig::kContainSecondaryNormal() const
{
    return d->kContainSecondaryNormal;
}

QColor GlobalDTConfig::kBrandHover() const
{
    return d->kBrandHover;
}

QColor GlobalDTConfig::kContainGeneralAlphaHover() const
{
    return d->kContainGeneralAlphaHover;
}

QColor GlobalDTConfig::kFontPrimary() const
{
    return d->kFontPrimary;
}

QColor GlobalDTConfig::kBrandNormal() const
{
    return d->kBrandNormal;
}

int GlobalDTConfig::normalline() const
{
    return d->normalline;
}

int GlobalDTConfig::focusline() const
{
    return d->focusline;
}

int GlobalDTConfig::kmarginMin() const
{
    return d->kmarginMin;
}

int GlobalDTConfig::kmarginNormal() const
{
    return d->kmarginNormal;
}

int GlobalDTConfig::kmarginBig() const
{
    return d->kmarginBig;
}

int GlobalDTConfig::kmarginContent() const
{
    return d->kmarginContent;
}

int GlobalDTConfig::kmagrinWindow() const
{
    return d->kmagrinWindow;
}

int GlobalDTConfig::kradiusMin() const
{
    return d->kradiusMin;
}

int GlobalDTConfig::kradiusMenu() const
{
    return d->kradiusMenu;
}

int GlobalDTConfig::kradiusNormal() const
{
    return d->kradiusNormal;
}

int GlobalDTConfig::kradiusWindow() const
{
    return d->kradiusWindow;
}


}
