/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

#ifndef DTTMPREADCONFIG_H
#define DTTMPREADCONFIG_H

#include <QObject>
#include <QColor>
#include <QPalette>

namespace UKUIGlobalDTConfig {

class GlobalDTConfigPrivate;

class GlobalDTConfig : public QObject
{
    Q_OBJECT

    //QT palette
    Q_PROPERTY(QColor windowTextActive          READ windowTextActive           NOTIFY tokenChanged)
    Q_PROPERTY(QColor windowTextInactive        READ windowTextInactive         NOTIFY tokenChanged)
    Q_PROPERTY(QColor windowTextDisable         READ windowTextDisable          NOTIFY tokenChanged)
    Q_PROPERTY(QColor buttonActive              READ buttonActive               NOTIFY tokenChanged)
    Q_PROPERTY(QColor buttonInactive            READ buttonInactive             NOTIFY tokenChanged)
    Q_PROPERTY(QColor buttonDisable             READ buttonDisable              NOTIFY tokenChanged)
    Q_PROPERTY(QColor lightActive               READ lightActive                NOTIFY tokenChanged)
    Q_PROPERTY(QColor lightInactive             READ lightInactive              NOTIFY tokenChanged)
    Q_PROPERTY(QColor lightDisable              READ lightDisable               NOTIFY tokenChanged)
    Q_PROPERTY(QColor midlightActive            READ midlightActive             NOTIFY tokenChanged)
    Q_PROPERTY(QColor midlightInactive          READ midlightInactive           NOTIFY tokenChanged)
    Q_PROPERTY(QColor midlightDisable           READ midlightDisable            NOTIFY tokenChanged)
    Q_PROPERTY(QColor darkActive                READ darkActive                 NOTIFY tokenChanged)
    Q_PROPERTY(QColor darkInactive              READ darkInactive               NOTIFY tokenChanged)
    Q_PROPERTY(QColor darkDisable               READ darkDisable                NOTIFY tokenChanged)
    Q_PROPERTY(QColor midActive                 READ midActive                  NOTIFY tokenChanged)
    Q_PROPERTY(QColor midInactive               READ midInactive                NOTIFY tokenChanged)
    Q_PROPERTY(QColor midDisable                READ midDisable                 NOTIFY tokenChanged)
    Q_PROPERTY(QColor textActive                READ textActive                 NOTIFY tokenChanged)
    Q_PROPERTY(QColor textInactive              READ textInactive               NOTIFY tokenChanged)
    Q_PROPERTY(QColor textDisable               READ textDisable                NOTIFY tokenChanged)
    Q_PROPERTY(QColor brightTextActive          READ brightTextActive           NOTIFY tokenChanged)
    Q_PROPERTY(QColor brightTextInactive        READ brightTextInactive         NOTIFY tokenChanged)
    Q_PROPERTY(QColor brightTextDisable         READ brightTextDisable          NOTIFY tokenChanged)
    Q_PROPERTY(QColor buttonTextActive          READ buttonTextActive           NOTIFY tokenChanged)
    Q_PROPERTY(QColor buttonTextInactive        READ buttonTextInactive         NOTIFY tokenChanged)
    Q_PROPERTY(QColor buttonTextDisable         READ buttonTextDisable          NOTIFY tokenChanged)
    Q_PROPERTY(QColor baseActive                READ baseActive                 NOTIFY tokenChanged)
    Q_PROPERTY(QColor baseInactive              READ baseInactive               NOTIFY tokenChanged)
    Q_PROPERTY(QColor baseDisable               READ baseDisable                NOTIFY tokenChanged)
    Q_PROPERTY(QColor windowActive              READ windowActive               NOTIFY tokenChanged)
    Q_PROPERTY(QColor windowInactive            READ windowInactive             NOTIFY tokenChanged)
    Q_PROPERTY(QColor windowDisable             READ windowDisable              NOTIFY tokenChanged)
    Q_PROPERTY(QColor shadowActive              READ shadowActive               NOTIFY tokenChanged)
    Q_PROPERTY(QColor shadowInactive            READ shadowInactive             NOTIFY tokenChanged)
    Q_PROPERTY(QColor shadowDisable             READ shadowDisable              NOTIFY tokenChanged)
    Q_PROPERTY(QColor highLightActive           READ highLightActive            NOTIFY tokenChanged)
    Q_PROPERTY(QColor highLightInactive         READ highLightInactive          NOTIFY tokenChanged)
    Q_PROPERTY(QColor highLightDisable          READ highLightDisable           NOTIFY tokenChanged)
    Q_PROPERTY(QColor highLightedTextActive     READ highLightedTextActive      NOTIFY tokenChanged)
    Q_PROPERTY(QColor highLightedTextInactive   READ highLightedTextInactive    NOTIFY tokenChanged)
    Q_PROPERTY(QColor highLightedTextDisable    READ highLightedTextDisable     NOTIFY tokenChanged)
    Q_PROPERTY(QColor linkActive                READ linkActive                 NOTIFY tokenChanged)
    Q_PROPERTY(QColor linkInactive              READ linkInactive               NOTIFY tokenChanged)
    Q_PROPERTY(QColor linkDisable               READ linkDisable                NOTIFY tokenChanged)
    Q_PROPERTY(QColor linkVisitedActive         READ linkVisitedActive          NOTIFY tokenChanged)
    Q_PROPERTY(QColor linkVisitedInactive       READ linkVisitedInactive        NOTIFY tokenChanged)
    Q_PROPERTY(QColor linkVisitedDisable        READ linkVisitedDisable         NOTIFY tokenChanged)
    Q_PROPERTY(QColor alternateBaseActive       READ alternateBaseActive        NOTIFY tokenChanged)
    Q_PROPERTY(QColor alternateBaseInactive     READ alternateBaseInactive      NOTIFY tokenChanged)
    Q_PROPERTY(QColor alternateBaseDisable      READ alternateBaseDisable       NOTIFY tokenChanged)
    Q_PROPERTY(QColor noRoleActive              READ noRoleActive               NOTIFY tokenChanged)
    Q_PROPERTY(QColor noRoleInactive            READ noRoleInactive             NOTIFY tokenChanged)
    Q_PROPERTY(QColor noRoleDisable             READ noRoleDisable              NOTIFY tokenChanged)
    Q_PROPERTY(QColor toolTipBaseActive         READ toolTipBaseActive          NOTIFY tokenChanged)
    Q_PROPERTY(QColor toolTipBaseInactive       READ toolTipBaseInactive        NOTIFY tokenChanged)
    Q_PROPERTY(QColor toolTipBaseDisable        READ toolTipBaseDisable         NOTIFY tokenChanged)
    Q_PROPERTY(QColor toolTipTextActive         READ toolTipTextActive          NOTIFY tokenChanged)
    Q_PROPERTY(QColor toolTipTextInactive       READ toolTipTextInactive        NOTIFY tokenChanged)
    Q_PROPERTY(QColor toolTipTextDisable        READ toolTipTextDisable         NOTIFY tokenChanged)
    Q_PROPERTY(QColor placeholderTextActive     READ placeholderTextActive      NOTIFY tokenChanged)
    Q_PROPERTY(QColor placeholderTextInactive   READ placeholderTextInactive    NOTIFY tokenChanged)
    Q_PROPERTY(QColor placeholderTextDisable    READ placeholderTextDisable     NOTIFY tokenChanged)

    //custom palette
    Q_PROPERTY(QColor kBlack                        READ kBlack                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrand1                       READ kBrand1                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrand2                       READ kBrand2                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrand3                       READ kBrand3                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrand4                       READ kBrand4                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrand5                       READ kBrand5                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrand6                       READ kBrand6                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrand7                       READ kBrand7                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrandClick                   READ kBrandClick                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrandFocus                   READ kBrandFocus                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrandHover                   READ kBrandHover                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kBrandNormal                  READ kBrandNormal                   NOTIFY tokenChanged)
    Q_PROPERTY(QColor kComponentAlphaClick          READ kComponentAlphaClick           NOTIFY tokenChanged)
    Q_PROPERTY(QColor kComponentAlphaDisable        READ kComponentAlphaDisable         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kComponentAlphaHover          READ kComponentAlphaHover           NOTIFY tokenChanged)
    Q_PROPERTY(QColor kComponentAlphaNormal         READ kComponentAlphaNormal          NOTIFY tokenChanged)
    Q_PROPERTY(QColor kComponentClick               READ kComponentClick                NOTIFY tokenChanged)
    Q_PROPERTY(QColor kComponentDisable             READ kComponentDisable              NOTIFY tokenChanged)
    Q_PROPERTY(QColor kComponentHover               READ kComponentHover                NOTIFY tokenChanged)
    Q_PROPERTY(QColor kComponentNormal              READ kComponentNormal               NOTIFY tokenChanged)
    Q_PROPERTY(QColor kContainClick                 READ kContainClick                  NOTIFY tokenChanged)
    Q_PROPERTY(QColor kContainHover                 READ kContainHover                  NOTIFY tokenChanged)
    Q_PROPERTY(QColor kContainGeneralAlphaClick     READ kContainGeneralAlphaClick      NOTIFY tokenChanged)
    Q_PROPERTY(QColor kContainGeneralAlphaHover     READ kContainGeneralAlphaHover      NOTIFY tokenChanged)
    Q_PROPERTY(QColor kContainGeneralNormal         READ kContainGeneralNormal          NOTIFY tokenChanged)
    Q_PROPERTY(QColor kContainSecondaryAlpphaNormal READ kContainSecondaryAlpphaNormal  NOTIFY tokenChanged)
    Q_PROPERTY(QColor kContainSecondaryNormal       READ kContainSecondaryNormal        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kErrorClick                   READ kErrorClick                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kErrorHover                   READ kErrorHover                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kErrorNormal                  READ kErrorNormal                   NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontPrimary                  READ kFontPrimary                   NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontPrimaryDisable           READ kFontPrimaryDisable            NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontSecondary                READ kFontSecondary                 NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontSecondaryDisable         READ kFontSecondaryDisable          NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontStrong                   READ kFontStrong                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontWhite                    READ kFontWhite                     NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontWhiteDisable             READ kFontWhiteDisable              NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontWhiteSecondary           READ kFontWhiteSecondary            NOTIFY tokenChanged)
    Q_PROPERTY(QColor kFontWhiteSecondaryDisable    READ kFontWhiteSecondaryDisable     NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray0                        READ kGray0                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray1                        READ kGray1                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray2                        READ kGray2                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray3                        READ kGray3                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray4                        READ kGray4                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray5                        READ kGray5                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray6                        READ kGray6                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray7                        READ kGray7                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray8                        READ kGray8                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray9                        READ kGray9                         NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray10                       READ kGray10                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray11                       READ kGray11                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray12                       READ kGray12                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray13                       READ kGray13                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray14                       READ kGray14                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray15                       READ kGray15                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray16                       READ kGray16                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGray17                       READ kGray17                        NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha0                   READ kGrayAlpha0                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha1                   READ kGrayAlpha1                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha2                   READ kGrayAlpha2                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha3                   READ kGrayAlpha3                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha4                   READ kGrayAlpha4                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha5                   READ kGrayAlpha5                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha6                   READ kGrayAlpha6                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha7                   READ kGrayAlpha7                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha8                   READ kGrayAlpha8                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha9                   READ kGrayAlpha9                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha10                  READ kGrayAlpha10                   NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha11                  READ kGrayAlpha11                   NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha12                  READ kGrayAlpha12                   NOTIFY tokenChanged)
    Q_PROPERTY(QColor kGrayAlpha13                  READ kGrayAlpha13                   NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineBrandClick               READ kLineBrandClick                NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineBrandDisable             READ kLineBrandDisable              NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineBrandHover               READ kLineBrandHover                NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineBrandNormal              READ kLineBrandNormal               NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineComponentClick           READ kLineComponentClick            NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineComponentDisable         READ kLineComponentDisable          NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineComponentHover           READ kLineComponentHover            NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineComponentNormal          READ kLineComponentNormal           NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineDisable                  READ kLineDisable                   NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineNormal                   READ kLineNormal                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineWindow                   READ kLineWindow                    NOTIFY tokenChanged)
    Q_PROPERTY(QColor kLineWindowAcitive            READ kLineWindowAcitive             NOTIFY tokenChanged)
    Q_PROPERTY(QColor kModalmask                    READ kModalmask                     NOTIFY tokenChanged)
    Q_PROPERTY(QColor kSuccessNormal                READ kSuccessNormal                 NOTIFY tokenChanged)
    Q_PROPERTY(QColor kWarningNormal                READ kWarningNormal                 NOTIFY tokenChanged)
    Q_PROPERTY(QColor kWhite                        READ kWhite                         NOTIFY tokenChanged)


    //line and margin
    Q_PROPERTY(int normalline       READ normalline     NOTIFY tokenChanged)
    Q_PROPERTY(int focusline        READ focusline      NOTIFY tokenChanged)
    Q_PROPERTY(int kmarginMin       READ kmarginMin     NOTIFY tokenChanged)
    Q_PROPERTY(int kmarginNormal    READ kmarginNormal  NOTIFY tokenChanged)
    Q_PROPERTY(int kmarginBig       READ kmarginBig     NOTIFY tokenChanged)
    Q_PROPERTY(int kmarginContent   READ kmarginBig     NOTIFY tokenChanged)
    Q_PROPERTY(int kmagrinWindow    READ kmagrinWindow  NOTIFY tokenChanged)

    //radius
    Q_PROPERTY(int kradiusMin       READ kradiusMin     NOTIFY tokenChanged)
    Q_PROPERTY(int kradiusMenu      READ kradiusMenu    NOTIFY tokenChanged)
    Q_PROPERTY(int kradiusNormal    READ kradiusNormal  NOTIFY tokenChanged)
    Q_PROPERTY(int kradiusWindow    READ kradiusWindow  NOTIFY tokenChanged)

public:
    explicit GlobalDTConfig();
    ~GlobalDTConfig();

    //QT palette
    QColor windowTextActive() const;
    QColor windowTextInactive() const;
    QColor windowTextDisable() const;
    QColor buttonActive() const;
    QColor buttonInactive() const;
    QColor buttonDisable() const;
    QColor lightActive() const;
    QColor lightInactive() const;
    QColor lightDisable() const;
    QColor midlightActive() const;
    QColor midlightInactive() const;
    QColor midlightDisable() const;
    QColor darkActive() const;
    QColor darkInactive() const;
    QColor darkDisable() const;
    QColor midActive() const;
    QColor midInactive() const;
    QColor midDisable() const;
    QColor textActive() const;
    QColor textInactive() const;
    QColor textDisable() const;
    QColor brightTextActive() const;
    QColor brightTextInactive() const;
    QColor brightTextDisable() const;
    QColor buttonTextActive() const;
    QColor buttonTextInactive() const;
    QColor buttonTextDisable() const;
    QColor baseActive() const;
    QColor baseInactive() const;
    QColor baseDisable() const;
    QColor windowActive() const;
    QColor windowInactive() const;
    QColor windowDisable() const;
    QColor shadowActive() const;
    QColor shadowInactive() const;
    QColor shadowDisable() const;
    QColor highLightActive() const;
    QColor highLightInactive() const;
    QColor highLightDisable() const;
    QColor highLightedTextActive() const;
    QColor highLightedTextInactive() const;
    QColor highLightedTextDisable() const;
    QColor linkActive() const;
    QColor linkInactive() const;
    QColor linkDisable() const;
    QColor linkVisitedActive() const;
    QColor linkVisitedInactive() const;
    QColor linkVisitedDisable() const;
    QColor alternateBaseActive() const;
    QColor alternateBaseInactive() const;
    QColor alternateBaseDisable() const;
    QColor noRoleActive() const;
    QColor noRoleInactive() const;
    QColor noRoleDisable() const;
    QColor toolTipBaseActive() const;
    QColor toolTipBaseInactive() const;
    QColor toolTipBaseDisable() const;
    QColor toolTipTextActive() const;
    QColor toolTipTextInactive() const;
    QColor toolTipTextDisable() const;
    QColor placeholderTextActive() const;
    QColor placeholderTextInactive() const;
    QColor placeholderTextDisable() const;

    //custom palette
    QColor kBlack() const;
    QColor kBrand1() const;
    QColor kBrand2() const;
    QColor kBrand3() const;
    QColor kBrand4() const;
    QColor kBrand5() const;
    QColor kBrand6() const;
    QColor kBrand7() const;
    QColor kBrandClick() const;
    QColor kBrandFocus() const;
    QColor kBrandHover() const;
    QColor kBrandNormal() const;
    QColor kComponentAlphaClick() const;
    QColor kComponentAlphaDisable() const;
    QColor kComponentAlphaHover() const;
    QColor kComponentAlphaNormal() const;
    QColor kComponentClick() const;
    QColor kComponentDisable() const;
    QColor kComponentHover() const;
    QColor kComponentNormal() const;
    QColor kContainClick() const;
    QColor kContainHover() const;
    QColor kContainGeneralAlphaClick() const;
    QColor kContainGeneralAlphaHover() const;
    QColor kContainGeneralNormal() const;
    QColor kContainSecondaryAlpphaNormal() const;
    QColor kContainSecondaryNormal() const;
    QColor kErrorClick() const;
    QColor kErrorHover() const;
    QColor kErrorNormal() const;
    QColor kFontPrimary() const;
    QColor kFontPrimaryDisable() const;
    QColor kFontSecondary() const;
    QColor kFontSecondaryDisable() const;
    QColor kFontStrong() const;
    QColor kFontWhite() const;
    QColor kFontWhiteDisable() const;
    QColor kFontWhiteSecondary() const;
    QColor kFontWhiteSecondaryDisable() const;
    QColor kGray0() const;
    QColor kGray1() const;
    QColor kGray2() const;
    QColor kGray3() const;
    QColor kGray4() const;
    QColor kGray5() const;
    QColor kGray6() const;
    QColor kGray7() const;
    QColor kGray8() const;
    QColor kGray9() const;
    QColor kGray10() const;
    QColor kGray11() const;
    QColor kGray12() const;
    QColor kGray13() const;
    QColor kGray14() const;
    QColor kGray15() const;
    QColor kGray16() const;
    QColor kGray17() const;
    QColor kGrayAlpha0() const;
    QColor kGrayAlpha1() const;
    QColor kGrayAlpha2() const;
    QColor kGrayAlpha3() const;
    QColor kGrayAlpha4() const;
    QColor kGrayAlpha5() const;
    QColor kGrayAlpha6() const;
    QColor kGrayAlpha7() const;
    QColor kGrayAlpha8() const;
    QColor kGrayAlpha9() const;
    QColor kGrayAlpha10() const;
    QColor kGrayAlpha11() const;
    QColor kGrayAlpha12() const;
    QColor kGrayAlpha13() const;
    QColor kLineBrandClick() const;
    QColor kLineBrandDisable() const;
    QColor kLineBrandHover() const;
    QColor kLineBrandNormal() const;
    QColor kLineComponentClick() const;
    QColor kLineComponentDisable() const;
    QColor kLineComponentHover() const;
    QColor kLineComponentNormal() const;
    QColor kLineDisable() const;
    QColor kLineNormal() const;
    QColor kLineWindow() const;
    QColor kLineWindowAcitive() const;
    QColor kModalmask() const;
    QColor kSuccessNormal() const;
    QColor kWarningNormal() const;
    QColor kWhite() const;

    //line and margin
    int normalline() const;
    int focusline() const;
    int kmarginMin() const;
    int kmarginNormal() const;
    int kmarginBig() const;
    int kmarginContent() const;
    int kmagrinWindow() const;

    //radius
    int kradiusMin() const;
    int kradiusMenu() const;
    int kradiusNormal() const;
    int kradiusWindow() const;

Q_SIGNALS:
    void tokenChanged();

private:

    GlobalDTConfigPrivate *d {nullptr};
};

}
#endif // READCONFIG_H
